#!/usr/bin/perl

##############################################################################
#  Copyright (C) 2007 Paulo Cesar Pereira de Andrade.  All Rights Reserved.
#
#  This is free software; you can redistribute it and/or modify
#  it under the terms of the GNU General Public License as published by
#  the Free Software Foundation; either version 2 of the License, or
#  (at your option) any later version.
#
#  This software is distributed in the hope that it will be useful,
#  but WITHOUT ANY WARRANTY; without even the implied warranty of
#  MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
#  GNU General Public License for more details.
#
#  Authors:
#	Paulo Cesar Pereira de Andrade <pcpa@mandriva.com.br>
##############################################################################

use File::Basename;

use strict;

my $x_depsdir = "/usr/share/X11/mandriva/";
my %dependencies;
my %rpms;

# hand build table until xorg modules match rpm packages
my %trans =
(
    'appres-1.0.1'		=> "appres",
    'bdftopcf-1.0.1'		=> "bdftopcf",
    'beforelight-1.0.3'		=> "beforelight",
    'bitmap-1.0.3'		=> "bitmap",
    'compiz-0.6.2'		=> "compiz",
    'editres-1.0.3'		=> "editres",
    'fonttosfnt-1.0.3'		=> "fonttosfnt",
    'fslsfonts-1.0.1'		=> "fslsfonts",
    'fstobdf-1.0.2'		=> "fstobdf",
    'grandr-0.1'		=> "grandr",
    'iceauth-1.0.2'		=> "iceauth",
    'ico-1.0.2'			=> "ico",
    'lbxproxy-1.0.1'		=> "lbxproxy",
    'libAppleWM-1.0.0'		=> "libapplewm-devel",
    'libdmx-1.0.2'		=> "libdmx-devel",
    'libfontenc-1.0.4'		=> "libfontenc1-devel",
    'libFS-1.0.0'		=> "libfs-devel",
    'libICE-1.0.4'		=> "libice6-devel",
    'liblbxutil-1.0.1'		=> "liblbxutil-devel",
    'liboldX-1.0.1'		=> "liboldx6-devel",
    'libpciaccess-0.9.1'	=> "libpciaccess-devel",
    'libSM-1.0.3'		=> "libsm6-devel",
    'libWindowsWM-1.0.0'	=> "libwindowswm-devel",
    'libX11-1.1.3'		=> "libx11_6-devel",
    'libXau-1.0.3'		=> "libxau6-devel",
    'libXaw-1.0.4'		=> "libxaw-devel",
    'libXcomposite-0.4.0'	=> "libxcomposite1-devel",
    'libXcursor-1.1.9'		=> "libxcursor-devel",
    'libXdamage-1.1.1'		=> "libxdamage-devel",
    'libXdmcp-1.0.2'		=> "libxdmcp6-devel",
    'libXevie-1.0.2'		=> "libxevie-devel",
    'libXext-1.0.3'		=> "libxext6-devel",
    'libXfixes-4.0.3'		=> "libxfixes3-devel",
    'libXfont-1.3.1'		=> "libxfont1-devel",
    'libXfontcache-1.0.4'	=> "libxfontcache1-devel",
    'libXft-2.1.12'		=> "libxft-devel",
    'libXi-1.1.3'		=> "libxi-devel",
    'libXinerama-1.0.2'		=> "libxinerama1-devel",
    'libxkbfile-1.0.4'		=> "libxkbfile-devel",
    'libXmu-1.0.3'		=> "libxmu6-devel",
    'libXp-1.0.0'		=> "libxp-devel",
    'libXpm-3.5.7'		=> "libxpm-devel",
    'libXprintAppUtil-1.0.1'	=> "libxprintapputil-devel",
    'libXprintUtil-1.0.1'	=> "libxprintutil-devel",
    'libXrandr-1.2.2'		=> "libxrandr2-devel",
    'libXrender-0.9.4'		=> "libxrender1-devel",
    'libXres-1.0.3'		=> "libxres1-devel",
    'libXScrnSaver-1.1.2'	=> "libxscrnsaver1-devel",
    'libXt-1.0.5'		=> "libxt6-devel",
    'libXTrap-1.0.0'		=> "libxtrap-devel",
    'libXtst-1.0.3'		=> "libxtst6-devel",
    'libXv-1.0.3'		=> "libxv1-devel",
    'libXvMC-1.0.4'		=> "libxvmc1-devel",
    'libXxf86dga-1.0.2'		=> "libxxf86dga-devel",
    'libXxf86misc-1.0.1'	=> "libxxf86misc-devel",
    'libXxf86vm-1.0.1'		=> "libxxf86vm-devel",
    'listres-1.0.1'		=> "listres",
    'luit-1.0.2'		=> "luit",
    'mkcfm-1.0.1'		=> "mkcfm",	# broken
    'mkcomposecache-1.2'	=> "mkcomposecache",
    'mkfontdir-1.0.3'		=> "mkfontdir",
    'mkfontscale-1.0.3'		=> "mkfontscale",
    'oclock-1.0.1'		=> "oclock",
    'proxymngr-1.0.1'		=> "proxymngr",
    'rgb-1.0.1'			=> "rgb",
    'rstart-1.0.2'		=> "rstart",
    'sessreg-1.0.3'		=> "sessreg",
    'setxkbmap-1.0.4'		=> "setxkbmap",
    'showfont-1.0.1'		=> "showfont",
    'twm-1.0.3'			=> "twm",
    'viewres-1.0.1'		=> "viewres",
    'x11perf-1.4.1'		=> "x11perf",
    'xauth-1.0.2'		=> "xauth",
    'xbacklight-1.1'		=> "xbacklight",
    'xbiff-1.0.1'		=> "xbiff",
    'xcalc-1.0.2'		=> "xcalc",
    'xclipboard-1.0.1'		=> "xclipboard",
    'xclock-1.0.3'		=> "xclock",
    'xcmsdb-1.0.1'		=> "xcmsdb",
    'xcompmgr-1.1.3'		=> "xcompmgr",
    'xconsole-1.0.3'		=> "xconsole",
    'xcursorgen-1.0.2'		=> "xcursorgen",
    'xdbedizzy-1.0.2'		=> "xdbedizzy",
    'xditview-1.0.1'		=> "xditview",
    'xdm-1.1.6'			=> "xdm",
    'xdpyinfo-1.0.2'		=> "xdpyinfo",
    'xdriinfo-1.0.2'		=> "xdriinfo",
    'xedit-1.0.2'		=> "xedit",
    'xev-1.0.2'			=> "xev",
    'xeyes-1.0.1'		=> "xeyes",
    'xf86dga-1.0.2'		=> "xf86dga",
    'xfd-1.0.1'			=> "xfd",
    'xfindproxy-1.0.1'		=> "xfindproxy",
    'xfontsel-1.0.2'		=> "xfontsel",
    'xfs-1.0.5'			=> "xfs",
    'xfsinfo-1.0.1'		=> "xfsinfo",
    'xfwp-1.0.1'		=> "xfwp",
    'xgamma-1.0.2'		=> "xgamma",
    'xgc-1.0.1'			=> "xgc",
    'xhost-1.0.2'		=> "xhost",
    'xinit-1.0.7'		=> "xinit",
    'xkbcomp-1.0.3'		=> "xkbcomp",
    'xkbevd-1.0.2'		=> "xkbevd",
    'xkbprint-1.0.1'		=> "xkbprint",
    'xkbutils-1.0.1'		=> "xkbutils",
    'xkill-1.0.1'		=> "xkill",
    'xload-1.0.2'		=> "xload",
    'xlogo-1.0.1'		=> "xlogo",
    'xlsatoms-1.0.1'		=> "xlsatoms",
    'xlsclients-1.0.1'		=> "xlsclients",
    'xlsfonts-1.0.2'		=> "xlsfonts",
    'xmag-1.0.2'		=> "xmag",
    'xman-1.0.3'		=> "xman",
    'xmessage-1.0.2'		=> "xmessage",
    'xmh-1.0.1'			=> "xmh",
    'xmodmap-1.0.3'		=> "xmodmap",
    'xmore-1.0.1'		=> "xmore",
    'xorg-server-1.4'		=> "x11-server",
    'xphelloworld-1.0.1'	=> "xphelloworld",
    'xplsprinters-1.0.1'	=> "xplsprinters",
    'xpr-1.0.2'			=> "xpr",
    'xprehashprinterlist-1.0.1'	=> "xprehashprinterlist",
    'xprop-1.0.3'		=> "xprop",
    'xrandr-1.2.2'		=> "xrandr",
    'xrdb-1.0.4'		=> "xrdb",
    'xrefresh-1.0.2'		=> "xrefresh",
    'xrestop-0.4'		=> "xrestop",
    'xrx-1.0.1'			=> "xrx",
    'xset-1.0.3'		=> "xset",
    'xsetmode-1.0.0'		=> "xsetmode",
    'xsetpointer-1.0.1'		=> "xsetpointer",
    'xsetroot-1.0.2'		=> "xsetroot",
    'xsm-1.0.1'			=> "xsm",
    'xstdcmap-1.0.1'		=> "xstdcmap",
    'xtrap-1.0.2'		=> "xtrap",
    'xvidtune-1.0.1'		=> "xvidtune",
    'xvinfo-1.0.2'		=> "xvinfo",
    'xwd-1.0.1'			=> "xwd",
    'xwininfo-1.0.3'		=> "xwininfo",
    'xwud-1.0.1'		=> "xwud",
);

##############################################################################
sub rpm_query {
    my ($name) = @_;

    $name = `rpm -q $name`;
    chomp($name);

    return $name;
};

##############################################################################
# Build dependencies hash table
foreach my $depfile (split("\n", `find $x_depsdir -name \\*-rpm.deps`)) {
    open(DEP, "<$depfile") or die("Cannot read $depfile ($!).");
    ## FIXME $depname could/should be the name of the rpm owner of the
    ## file, but at this time, still building the scripts...
    my $depname = File::Basename::basename($depfile);
    $depname =~ s/-rpm\.deps//;
    $depname = $trans{$depname};
    $rpms{$depname} = rpm_query($depname);
    while (<DEP>) {
	chomp;
	# remove comments
	s/\s+#.*//;
	my $shortname = $_;
	$shortname =~ s/-\d+(\.\d+).*$//;
	$rpms{$shortname} = $_;
	push(@{$dependencies{$depname}}, $shortname);
    }
    close(DEP);
}

##############################################################################
# Create list of dependencies in order that must be built to ensure
# everything is up to date. This ofcourse only checks dependencies on
# header files, data files or configuration files (like autotools m4 macros)
# must be properly updated in another way.
my %deplist;

# Don't check for possible cyclic dependencies, just order to build
# dependencies first.
# This isn't a reliable algorithm, but should be enough for a first version,
# to list proper order modules/packages should be built.
sub check_deps {
    my ($pkgname) = @_;

    my ($name, $list);

    # Make sure dependencies have a lower value
    foreach (@{$dependencies{$pkgname}}) {
	if ($deplist{$_} >= $deplist{$pkgname}) {
	    $deplist{$pkgname} = $deplist{$_} + 1;
	}
    }

    # Make sure everything that depends on $pkgname has a higher value
    while (($name, $list) = each(%dependencies)) {
	foreach (@$list) {
	    if ($_ eq $pkgname and $deplist{$name} <= $deplist{$pkgname}) {
		$deplist{$name} = $deplist{$pkgname} + 1;
	    }
	}
    }
}

# Initialize deplist
foreach (keys %dependencies) {
    $deplist{$_} = 0;
}

foreach (keys %dependencies) {
    check_deps($_);
}

foreach (sort{ $deplist{$a} <=> $deplist{$b} } keys(%deplist)) {
    printf("%-32s => %s\n", $_, $deplist{$_});
}
